import sys
import logging
from io import StringIO, BytesIO

from rocolib.api.composables.graph.Graph import Graph as BaseGraph
from rocolib.api.composables.graph.Drawing import Drawing
from rocolib.api.composables.Composable import Composable
from rocolib.utils.utils import decorateGraph
from rocolib.utils.tabs import BeamTabs, BeamTabDecoration, BeamSlotDecoration
import rocolib.utils.numsym as np


log = logging.getLogger(__name__)

class DecorationComposable(Composable, BaseGraph):
  def __init__(self):
    BaseGraph.__init__(self)
  def append(self, newComposable, newPrefix, **kwargs):
    pass
  def attach(self, fromInterface, toInterface, kwargs):
    pass
  def makeOutput(self, filedir, **kwargs):
    pass

class GraphComposable(Composable, BaseGraph):
  def __init__(self):
    BaseGraph.__init__(self)

  def append(self, g2, prefix2, **kwargs):
    if kwargs.get("invert", False):
        g2.invertEdges()
    g2.prefix(prefix2)

    if kwargs.get("root", False):
        self.faces = g2.faces + self.faces
        self.edges = g2.edges + self.edges
    else:
        self.faces.extend(g2.faces)
        self.edges.extend(g2.edges)


  def makeOutput(self, filedir, **kwargs):
    if "displayOnly" in kwargs:
      kwDefault = not kwargs["displayOnly"]
      kwargs["display"] = kwargs["displayOnly"]
    elif "default" in kwargs:
      kwDefault = kwargs["default"]
    else:
      kwDefault = True

    def kw(arg, default=kwDefault):
      if arg in kwargs:
        return kwargs[arg]
      return default

    self.tabify(kw("tabFace", BeamTabs), kw("tabDecoration", BeamTabDecoration),
                kw("slotFace", None), kw("slotDecoration", BeamSlotDecoration), **kwargs)
    if kw("joint", None):
        self.jointify(**kwargs)
    self.place(transform3D=kw("transform3D", None))

    basename = None
    if filedir:
        basename = filedir + "/" + kw("name", "") + "-"

    rets = {}
    bufs = {}

    def handle(keyword, text, fn, stub, **ka):
        if kw(keyword):
          log.info("Generating %s pattern..." % text)
          sys.stdout.flush()
          if basename and stub:
            with open(basename+stub, 'w') as fp:
              fn(fp, **ka)
          else:
            buf = StringIO()
            fn(buf, **ka)
            rets[keyword] = buf.getvalue()
            bufs[keyword] = buf
            
          log.debug("Done generating %s pattern." % text)

    d = Drawing()
    if kw("display", False) or kw("unfolding") or kw("autofolding") or kw("silhouette") or kw("animate"):
      d.fromGraph(self)
      d.transform(relative=(0,0))

    handle("unfolding", "Corel cut-and-fold", d.toSVG, "lasercutter.svg", mode="Corel")
    handle("unfolding", "printer", d.toSVG, "lasercutter.svg", mode="print")
    handle("animate", "OrigamiSimulator", d.toSVG, "anim.svg", mode="animate")
    handle("silhouette", "Silhouette cut-and-fold", d.toDXF, "silhouette.dxf", mode="silhouette")
    handle("autofolding", "autofolding", d.toDXF, "autofold-default.dxf", mode="autofold")
    handle("autofolding", "  -- (graph)", d.toDXF, "autofold-graph.dxf")
    handle("stl", "stl", self.to3D, "model.stl", format="stl", **kwargs)
    handle("webots", "webots", self.to3D, "model.wbo", format="webots", **kwargs)


    if kw("display3D", False) or kw("png"):
      from rocolib.utils.display import display3D

      if basename:
        if kw("stl"):
          try:
            stl_handle = open(basename+"model.stl", 'rb')
          except:
            log.info("No Stl found")
            return
        else:
          log.info("STL wasn't selected, making now...")
          handle("png", "stl", self.to3D, None, format="stl")
          stl_handle = bufs['png']
          stl_handle.seek(0)
        if kw("png"):
          png_file = basename+"model.png"
      else:
        stl_handle = bufs['stl']
        stl_handle.seek(0)
        png_file = BytesIO()

      with stl_handle as sh:
        display3D(sh, png_file, kw("display3D", False))

      if kw("png") and not basename:
        rets["png"] = png_file.getvalue()

    if kw("display", False):
      from rocolib.utils.display import displayTkinter
      displayTkinter(d)

    return rets
